
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ttversion.h"
#include "revtools.h"
#include "tt.h"

//=============================================================================
// outputs usage information of this tool
//=============================================================================
void PrintUsage() {
    PRINT_ID("TTBin2Asm");
    fprintf(USAGE_OUT, "Usage: ttbin2asm [flags] <infile> <outfile>\n\n"              \
                       "       -quiet      ....  don't output standard messages\n"    \
                       "       -slash      ....  append backslash at end of line\n"   \
                       "       -t <text>   ....  add text to beginning of file\n"     \
                       "       -a <name>   ....  generate an array with given name\n" \
                       "       -ss <nr>    ....  skip number of bytes at begin of inputfile\n"\
                       "       -se <nr>    ....  skip number of bytes at end of inputfile\n"\
                       "       -lb <nr>    ....  insert an additionally linebreak every nr items\n"\
                       "       infile      ....  name of infile\n"                    \
                       "       outfile     ....  name of outfile (use \"-\" for stdout)\n\n"  \
                       "       converts binary file to ASM .text\n\n");
}


//=============================================================================
// ... another "just a main routine" tool ...
//=============================================================================
int main(int argc,char *argv[]) {
    char*          infile    = NULL;
    char*          outfile   = NULL;
    FILE*          ifp;
    FILE*          ofp;
    int            bytes_per_item = 1;
    int            n;
    int            data[4];
    int            appendslash = 0;
    long           insize;
    long           bytes_processed;
    long           loop;
    long           itemcount;
    char*          arrayname  = NULL;
    char*          starttext  = NULL;
    int            quiet      = 0;
    int            skip_start = 0;
    int            skip_end   = 0;
    int            empty_line = 0;

    // check for too less arguments
    if (argc < 3) {
        PrintUsage();
        return 1;
    }


    // parse arguments
    for (n=1; n<argc; n++) {
        if (!strcmp(argv[n], "-slash"))      appendslash    = 1;
        else if (!strcmp(argv[n], "-quiet")) quiet          = 1;
        else if (!strcmp(argv[n], "-ss")) {
            if (n == argc -1) {
                PrintUsage();
                return 1;
            }
            else {
                if (sscanf(argv[n+1],"%d",&skip_start) != 1) {
                    PrintUsage();
                    return 1;
                }
                if (skip_start < 0) {
                    fprintf(stderr,"ERROR: invalid count for option -ss\n");
                    return 1;
                }
                n++;
            }
        }

        else if (!strcmp(argv[n], "-se")) {
            if (n == argc -1) {
                PrintUsage();
                return 1;
            }
            else {
                if (sscanf(argv[n+1],"%d",&skip_end) != 1) {
                    PrintUsage();
                    return 1;
                }
                if (skip_end < 0) {
                    fprintf(stderr,"ERROR: invalid count for option -se\n");
                    return 1;
                }
                n++;
            }
        }

        else if (!strcmp(argv[n], "-a")) {
            if (n == argc -1) {
                PrintUsage();
                return 1;
            }
            else {
                arrayname = argv[n+1];
                n++;
            }
        }
        
        else if (!strcmp(argv[n], "-t")) {
            if (n == argc -1) {
                PrintUsage();
                return 1;
            }
            else {
                starttext = argv[n+1];
                n++;
            }
        }

        else if (!strcmp(argv[n], "-lb")) {
            if (n == argc -1) {
                PrintUsage();
                return 1;
            }
            else {
                if (sscanf(argv[n+1],"%d",&empty_line) != 1) {
                    PrintUsage();
                    return 1;
                }
                n++;
            }
        }

        else if (!infile)  infile  = argv[n];
        else if (!outfile) outfile = argv[n];
        else {
            PrintUsage();
            return 1;
        }
    }

    if (!infile || !outfile) {
        PrintUsage();
        return 1;
    }

    if (!(ifp = fopen(infile,"rb"))) {
        fprintf(stderr,"ERROR: cannot open inputfile %s\n",infile);
        return 1;
    }

    if (!strcmp(outfile,"-")) {
       ofp = stdout;
    }
    else {
        if (!quiet) PRINT_ID("TTBin2Asm");
        if (!(ofp = fopen(outfile,"w"))) {
            fprintf(stderr,"ERROR: cannot open outputfile %s\n",outfile);
            fclose(ifp);
            return 1;
        }
    }

    fseek(ifp,0,SEEK_END);
    insize = ftell(ifp);
    rewind(ifp);

    bytes_processed = 0;

    insize-=skip_start;
    insize-=skip_end;

    if (insize<0) {
        fprintf(stderr,"ERROR: no bytes left to process\n");
        fclose(ifp);
        fclose(ofp);
        return 1;
    }

    if (skip_start > 0) fseek(ifp,skip_start,SEEK_SET);

    if (starttext) {
        fprintf(ofp,"    %s\n",starttext);
    }

    if (arrayname) {
        fprintf(ofp,"        .text\n.globl	%s\n%s:\n",arrayname,arrayname);
    }

    for (loop = 0;loop<insize;loop++) {
        data[0] = fgetc(ifp);
        fprintf(ofp,"        .byte 0x%02x\n", data[0]);

        bytes_processed++;

        if (empty_line) {
            if (!((loop+1) % empty_line)) {
                if (appendslash) fputc('\\',ofp);
                fputc('\n',ofp);
            }
        }


    }


    fputc('\n',ofp);

    fclose(ifp);
    if (ofp != stdout) {
        fclose(ofp);
        loop = insize;
        if (!quiet) fprintf(stdout,"%ld items with 1 byte written to %s\n",loop,outfile);
    }

    return 0;
}

//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: ttbin2hex.c,v $
// Revision 1.9  2002/05/07 15:53:38  tnussb
// (1) option -a fixed. previously the array length was off by one when the
//     input length was not a multiple of the itemsize
// (2) option -lb added. Now additionally line breaks can be added after every
//     n items.
//
// Revision 1.8  2002/04/17 10:46:45  tnussb
// commandline options -ss and -se added
//
// Revision 1.7  2002/03/15 13:17:30  tnussb
// (1) treatment of -quiet (suppress standard messages) added
//
// Revision 1.6  2002/02/07 09:49:36  tnussb
// all local includes changed, because header files are now located in pctools folder
//
// Revision 1.5  2001/06/20 13:08:01  Thomas Nussbaumer
// (1) handling of output redirection to stdout (outfilename == "-")
// (2) handling of "-a arrayname" to generate a complete C array of name arrayname
//
// Revision 1.4  2001/02/05 20:42:49  Thomas Nussbaumer
// (1) adding no comma after last item anymore
// (2) -slash parameter implemented (adding a backslash at end of line)
//
// Revision 1.3  2000/11/28 00:03:04  Thomas Nussbaumer
// using now USAGE_OUT stream for usage info
//
// Revision 1.2  2000/08/23 19:35:47  Thomas Nussbaumer
// adapted to automatic version display (revtools.h)
//
// Revision 1.1  2000/08/13 20:20:55  Thomas Nussbaumer
// initial version
//
//
//
//
